@@ -171,11 +171,11 @@ unsigned long _dl_linux_resolver(struct elf_resolve *tpnt, int reloc_entry)
 	Elf32_Sym *symtab;
 	ELF_RELOC *rel_addr;
 	int symtab_index;
+	char *symname;
 	unsigned long insn_addr;
 	unsigned long *insns;
 	unsigned long new_addr;
-	char *symname;
-	int delta;
+	unsigned long delta;
 
 	rel_addr = (ELF_RELOC *) (tpnt->dynamic_info[DT_JMPREL] + tpnt->loadaddr);
 
@@ -231,39 +231,36 @@ unsigned long _dl_linux_resolver(struct elf_resolve *tpnt, int reloc_entry)
 	}else{
 		/* Warning: we don't handle double-sized PLT entries */
 		unsigned long plt_addr;
-		unsigned long lbranch_addr;
 		unsigned long *ptr;
 		int index;
 
 		plt_addr = (unsigned long)tpnt->dynamic_info[DT_PLTGOT] + 
 			(unsigned long)tpnt->loadaddr;
-		lbranch_addr = plt_addr + PLT_LONGBRANCH_ENTRY_WORDS*4;
-		delta = lbranch_addr - insn_addr;
+
+		delta = PLT_LONGBRANCH_ENTRY_WORDS*4 - (insn_addr-plt_addr+4);
+
 		index = (insn_addr - plt_addr - PLT_INITIAL_ENTRY_WORDS*4)/8;
 
 		ptr = (unsigned long *)tpnt->data_words;
 		//DPRINTF("plt_addr=%x delta=%x index=%x ptr=%x\n", plt_addr, delta, index, ptr);
+		insns += 1;
+
 		ptr[index] = new_addr;
+		PPC_SYNC;
 		/* icache sync is not necessary, since this will be a data load */
 		//PPC_DCBST(ptr+index);
 		//PPC_SYNC;
 		//PPC_ICBI(ptr+index);
 		//PPC_ISYNC;
 
-		/* instructions were modified */
-		insns[1] = OPCODE_B(delta - 4);
-		PPC_DCBST(insn_addr+1);
-		PPC_SYNC;
-		PPC_ICBI(insn_addr+1);
-		PPC_ISYNC;
-		
-		return new_addr;
+		insns[0] = OPCODE_B(delta);
+
 	}
 
 	/* instructions were modified */
-	PPC_DCBST(insn_addr);
+	PPC_DCBST(insns);
 	PPC_SYNC;
-	PPC_ICBI(insn_addr);
+	PPC_ICBI(insns);
 	PPC_ISYNC;
 
 	return new_addr;
@@ -360,7 +357,8 @@ _dl_do_lazy_reloc (struct elf_resolve *tpnt, struct dyn_elf *scope,
 			break;
 		case R_PPC_JMP_SLOT:
 			{
-				int index, delta;
+				int index;
+				unsigned long delta;
 				unsigned long *plt;
 				unsigned long *insns;
 
@@ -393,10 +391,10 @@ _dl_do_lazy_reloc (struct elf_resolve *tpnt, struct dyn_elf *scope,
 
 	/* instructions were modified */
 	PPC_DCBST(reloc_addr);
-	PPC_DCBST(reloc_addr+1);
+	PPC_DCBST(reloc_addr+4);
 	PPC_SYNC;
 	PPC_ICBI(reloc_addr);
-	PPC_ICBI(reloc_addr+1);
+	PPC_ICBI(reloc_addr+4);
 	PPC_ISYNC;
 
 #if defined (__SUPPORT_LD_DEBUG__)
@@ -429,7 +427,7 @@ _dl_do_reloc (struct elf_resolve *tpnt,struct dyn_elf *scope,
 	if (symtab_index) {
 
 		symbol_addr = (unsigned long) _dl_find_hash(symname, scope, 
-				(reloc_type == R_386_JMP_SLOT ? tpnt : NULL), symbolrel);
+				(reloc_type == R_PPC_JMP_SLOT ? tpnt : NULL), symbolrel);
 
 		/*
 		 * We want to allow undefined references to weak symbols - this might
@@ -456,7 +454,7 @@ _dl_do_reloc (struct elf_resolve *tpnt,struct dyn_elf *scope,
 			case R_PPC_REL24:
 #if 0
 				{
-					int delta = symbol_addr - (unsigned long)reloc_addr;
+					unsigned long delta = symbol_addr - (unsigned long)reloc_addr;
 					if(delta<<6>>6 != delta){
 						_dl_dprintf(2,"R_PPC_REL24: Reloc out of range\n");
 						_dl_exit(1);
@@ -489,31 +487,34 @@ _dl_do_reloc (struct elf_resolve *tpnt,struct dyn_elf *scope,
 			case R_PPC_JMP_SLOT:
 				{
 					unsigned long targ_addr = (unsigned long)*reloc_addr;
-					int delta = targ_addr - (unsigned long)reloc_addr;
+					unsigned long delta = targ_addr - (unsigned long)reloc_addr;
 					if(delta<<6>>6 == delta){
 						*reloc_addr = OPCODE_B(delta);
 					}else if (targ_addr <= 0x01fffffc || targ_addr >= 0xfe000000){
 						*reloc_addr = OPCODE_BA (targ_addr);
 					}else{
 						{
-							int delta;
 							int index;
-							unsigned long *plt;
+							unsigned long delta2;
+							unsigned long *plt, *ptr;
 							plt = (unsigned long *)(tpnt->dynamic_info[DT_PLTGOT] + tpnt->loadaddr);
 
-							delta = (unsigned long)(plt+PLT_TRAMPOLINE_ENTRY_WORDS+2)
+							delta2 = (unsigned long)(plt+PLT_LONGBRANCH_ENTRY_WORDS)
 								- (unsigned long)(reloc_addr+1);
 
 							index = ((unsigned long)reloc_addr -
 									(unsigned long)(plt+PLT_INITIAL_ENTRY_WORDS))
 								/sizeof(unsigned long);
 							index /= 2;
-							//DPRINTF("        index %x delta %x\n",index,delta);
+							//DPRINTF("        index %x delta %x\n",index,delta2);
+							ptr = (unsigned long *)tpnt->data_words;
+							ptr[index] = targ_addr;
 							reloc_addr[0] = OPCODE_LI(11,index*4);
-							reloc_addr[1] = OPCODE_B(delta);
+							reloc_addr[1] = OPCODE_B(delta2);
 
 							/* instructions were modified */
 							PPC_DCBST(reloc_addr+1);
+							PPC_SYNC;
 							PPC_ICBI(reloc_addr+1);
 						}
 					}
@@ -572,7 +573,7 @@ _dl_do_copy (struct elf_resolve *tpnt, struct dyn_elf *scope,
 	  
 	reloc_addr = (unsigned long *)(intptr_t) (tpnt->loadaddr + (unsigned long) rpnt->r_offset);
 	reloc_type = ELF32_R_TYPE(rpnt->r_info);
-	if (reloc_type != R_386_COPY) 
+	if (reloc_type != R_PPC_COPY) 
 		return 0;
 	symtab_index = ELF32_R_SYM(rpnt->r_info);
 	symbol_addr = 0;
@@ -589,8 +590,8 @@ _dl_do_copy (struct elf_resolve *tpnt, struct dyn_elf *scope,
 			     symname, symtab[symtab_index].st_size,
 			     symbol_addr, symtab[symtab_index].st_value);
 #endif
-		_dl_memcpy((char *) symtab[symtab_index].st_value, 
-			(char *) symbol_addr, symtab[symtab_index].st_size);
+			_dl_memcpy((char *) reloc_addr,
+					(char *) symbol_addr, symtab[symtab_index].st_size);
 	}
 
 	return goof;
