@@ -12,8 +12,12 @@
  */
 
 #define	_ERRNO_H
+#include <features.h>
 #include <unistd.h>
 #include <stdlib.h>
+#ifdef __UCLIBC_PROPOLICE__
+extern void __guard_setup(void);
+#endif
 
 
 /*
@@ -23,7 +27,6 @@ extern int  main(int argc, char **argv, char **envp);
 extern void weak_function _stdio_init(void);
 extern int *weak_const_function __errno_location(void);
 extern int *weak_const_function __h_errno_location(void);
-extern int weak_function atexit(void (*function)(void));
 #ifdef __UCLIBC_HAS_LOCALE__
 extern void weak_function _locale_init(void);
 #endif
@@ -42,6 +45,7 @@ extern void weak_function __pthread_initialize_minimal(void);
  */
 
 char **__environ = 0;
+const char *__progname = 0;
 weak_alias(__environ, environ);
 
 
@@ -100,6 +104,9 @@ void __uClibc_init(void)
 
 }
 
+#ifdef __UCLIBC_CTOR_DTOR__
+void (*__app_fini)(void) = NULL;
+#endif
 
 /* __uClibc_start_main is the new main stub for uClibc. This function is 
  * called from crt0 (version 0.9.16 or newer), after ALL shared libraries 
@@ -123,15 +130,21 @@ __uClibc_start_main(int argc, char **argv, char **envp,
      * __uClibc_init() regardless, to be sure the right thing happens. */
     __uClibc_init();
 
+    __progname = *argv;
+
+#ifdef __UCLIBC_CTOR_DTOR__
     /* Arrange for the application's dtors to run before we exit.  */
-    if (app_fini!=NULL && atexit) {
-	atexit (app_fini);
-    }
+	__app_fini = app_fini;
 
     /* Run all the application's ctors now.  */
     if (app_init!=NULL) {
 	app_init();
     }
+#endif
+
+#ifdef __UCLIBC_PROPOLICE__
+     __guard_setup ();
+#endif
 
     /* Note: It is possible that any initialization done above could
      * have resulted in errno being set nonzero, so set it to 0 before
